;===========================================================================================
; Progress.asm  GoAsm assembler
; Construction manuelle de Progress32 class
; Programm par AsmGges France
;===========================================================================================
; compile.bat
; ..\..\bin\GoRC /r, res\progress.rc
; ..\..\bin\GoAsm progress.asm
; ..\..\bin\GoLink progress.obj res\progress.res user32.dll kernel32.dll gdi32.dll 
;                                                                     comctl32.dll
;===========================================================================================

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  #include "..\..\inc\Windows.inc"
  #include "..\..\inc\all_api.inc"

  ID_FRANCE      = 17

  ID_GBOX        = 100
  ID_PROGRESSH   = 101
  ID_PROGRESSV   = 102
  ID_SPIN        = 103
  ID_EDIT        = 104


  UDM_SETRANGE32 = 046Fh
  UDM_SETPOS32   = 0471h

  PBS_VERTICAL   = 04h
  PBS_SMOOTH     = 01h

;===========================================================================================
; Dbut du code programme
; -----------------------
.code

 start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],ID_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,addr wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,addr ClassName,addr Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU \
                         +WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,addr msg,0,0,0
    or eax,eax
    jz >ExitProgram
    invoke TranslateMessage,addr msg
    invoke DispatchMessage,addr msg
    jmp <MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
WindowProc frame @hWnd, @uMsg, @wParam, @lParam
    uses ebx,esi,edi
 
    cmp D[@uMsg],WM_CLOSE
    jne >.wmcreate
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],addr MsgEnd,addr ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande fin d'application
        ; -------------------------
        invoke PostQuitMessage,0
        jmp >>.exit

    .wmcreate:
    cmp D[@uMsg],WM_CREATE
    jne >.wmhscroll
       ; Assure que la bibliothque de liaison dynamique de contrle commune est charge
       ; -------------------------------------------------------------------------------
        mov D[icex.dwSize],sizeof INITCOMMONCONTROLSEX
        mov D[icex.dwICC],0FFFFh
        invoke InitCommonControlsEx,addr icex

        ; Dfinir une fonte de texte
        ; --------------------------
        invoke CreateFont,8,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                       CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                      addr FontMS
        mov [hFontMS],eax                      ;fonte handle

        ; Construit tous les controles
        ; ----------------------------
        invoke CreateControls,[@hWnd]

        ; Valeur initiale de Spin pour EDIT
        ; ---------------------------------
        mov D[NewPos],10

        ; Rafraichit la valeur pour control
        ; ---------------------------------
        invoke ProgressPos,[@hWnd]
        jmp >>.exit

    .wmhscroll:
    cmp D[@uMsg],WM_HSCROLL 
    je  >
        cmp D[@uMsg],WM_VSCROLL
        jne >>.defwindowproc
            :
            mov eax,[@wParam]
            and eax,0FFFFh
            cmp eax,TB_THUMBPOSITION
            jne >
                mov eax,[@wParam]
                shr eax,16
                mov [NewPos],eax
                invoke ProgressPos,[@hWnd]
                jmp >>.exit
            :
            cmp eax,TB_THUMBTRACK
            jne >
                mov eax,[@wParam]
                shr eax,16
                mov [NewPos],eax
                invoke ProgressPos,[@hWnd]
                jmp >>.exit
            :
            cmp eax,SB_LINEUP
            jne >.sblinedown
                cmp D[NewPos],0
                je  >
                    dec D[NewPos]
                :
                invoke ProgressPos,[@hWnd]
                jmp >>.exit

            .sblinedown:
            cmp eax,SB_LINEDOWN
            jne >.exit
                cmp D[NewPos],20
                je  >
                    inc D[NewPos]
                :
                invoke ProgressPos,[@hWnd]
                jmp >.exit

    .defwindowproc:
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .exit

    mov eax,0
    ret
endf

;===========================================================================================
CreateControls frame @hWnd

    ; Cration de la fentre-enfant GroupBox
    ; --------------------------------------
    invoke Dlg,addr Button,addr GbGges,[@hWnd],4,2,286,20,ID_GBOX,BS_GROUPBOX,0

    ;=======================================================================================
    ; Cration de la fentre-enfant ProgressH
    ; ---------------------------------------
    invoke Dlg,addr Progress,addr NoText,[@hWnd],93,134,114,20,ID_PROGRESSH,0,\
                                                          WS_EX_DLGMODALFRAME

    ; Min/max ProgressH
    ; -----------------
    invoke SendMessage,eax,PBM_SETRANGE32,0,20

    ;---------------------------------------------------------------------------------------
    ; Cration de la fentre-enfant ProgressV
    ; ---------------------------------------
    invoke Dlg,addr Progress,addr NoText,[@hWnd],140,28,20,98,ID_PROGRESSV,PBS_VERTICAL \
                                          +PBS_SMOOTH,WS_EX_CLIENTEDGE+WS_EX_STATICEDGE
    ; Min/max ProgressV
    ; -----------------
    invoke SendMessage,eax,PBM_SETRANGE32,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant Edit
    ; ----------------------------------
    invoke Dlg,addr Edit,addr NoText,[@hWnd],131,166,38,20,ID_EDIT,ES_CENTER+ES_NUMBER \
                                                         +ES_READONLY,WS_EX_CLIENTEDGE

    ;---------------------------------------------------------------------------------------
    ; Cration de la fentre-enfant Spin
    ; ----------------------------------
    invoke Dlg,addr Spin,addr NoText,[@hWnd],131,190,38,16,ID_SPIN,UDS_SETBUDDYINT \
                                           +UDS_AUTOBUDDY+UDS_ARROWKEYS+UDS_HORZ,0
    ; Min/max Spin
    ; ------------
    invoke SendMessage,eax,UDM_SETRANGE32,0,20

    ret
endf

;===========================================================================================
ProgressPos frame @hWnd

    invoke SendDlgItemMessage,[@hWnd],ID_PROGRESSH,PBM_SETPOS,[NewPos],0
    invoke SendDlgItemMessage,[@hWnd],ID_PROGRESSV,PBM_SETPOS,[NewPos],0
    invoke SendDlgItemMessage,[@hWnd],ID_SPIN,UDM_SETPOS32,0,[NewPos]

    ret
endf

;===========================================================================================
; Cration d'une fentre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class    = adresse du nom de control class
; @AdrText  = adresse du control text
; @hParent  = handle de la fentre parent
; @X        = x-coordonne fentre-enfant
; @Y        = y-coordonne fentre-enfant
; @Xw       = largeur en pixels fentre-enfant
; @Yh       = hauteur en pixels fentre-enfant
; @Id       = identificateur du controle forme ID_XXX
; @Style    = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle  = window style tendu
;-------------------------------------------------------------------------------------------
; => eax = handle de la fenetre-enfant
;-------------------------------------------------------------------------------------------
Dlg frame @Class, @AdrText, @hParent, @X, @Y, @Xw, @Yh, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AdrText],eax,[@X],[@Y],[@Xw],[@Yh],\
                          [@hParent],[@Id],[wc.hInstance],NULL
    mov ebx,eax

    ; Valider la fonte standard MS Sans Serif pour le control
    ; -------------------------------------------------------
    invoke SendMessage,ebx,WM_SETFONT,[hFontMS],NULL
    mov eax,ebx

    ret
endf

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
.data

  Titre       db ' PROGRESS32 control',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' GoAsm AsmGges France @2006  ',0
  GbGges      db 'AsmGges Progress32',0
  NoText      db '',0

  FontMS      db 'MS Sans Serif',0
  Verdana     db 'Verdana',0

  Edit        db 'EDIT',0
  Button      db 'BUTTON',0
  Progress    db 'msctls_progress32',0
  Spin        db 'msctls_updown32',0

  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 300
  WinH        dd 244

  NewPos      dd ?

  hWnd        dd ?
  hFontMS     dd ?

  wc          WNDCLASS < 0, addr WindowProc, 0, 0, 0, 0, 0, \
                         COLOR_BTNFACE+1, 0, addr ClassName >
  msg         MSG                  <>
  icex        INITCOMMONCONTROLSEX <>

;===========================================================================================

